﻿
Public Class Form1

    ' ===================================================================================
    '   Form
    ' ===================================================================================
    Private EventsAreEnabled As Boolean = False
    Private RefreshNeeded As Boolean

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Text = AppTitleAndVersion()
        Load_INI()
        LoadSlotNames()
        SetBufferLength()
        InitAll()
        Refresh()
        LoadBufferFrom(GetBuffersPath() + "LastBuffer.csv")
        InitGraphics(pbox_Scope)
        ToolStrip1.Renderer = New ToolStripButtonRenderer
        pbox_Scope.Select()
        lbl_SlotName1.Focus()
        SetScopeParams()
        ShowStatusBarValues()
    End Sub

    Friend Sub InitAll()
        EventsAreEnabled = True
        ModuleScope.LargeMarkers = btn_Time.Checked
        ModuleScope.InterpolateEnabled = btn_Interpolate.Checked
        ModuleScope.CursorsEnabled = btn_Cursors.Checked
        btn_Interpolate.Visible = Not btn_Cursors.Checked
        ModuleScope.SetDeltaTime(CSng(Val(txt_Delta.Text)))
        SetStopParams()
        SetSlotParams()
        SetScopeParams()
        UpdateTriggerButton()
        ShowHideControls()
        StartStopAcquisition(btn_Run.Checked)
        StartStopTimers()
        RefreshNeeded = True
    End Sub

    Private Sub Form1_VisibleChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.VisibleChanged, _
                                                                                                  Me.SizeChanged
        StartStopTimers()
        ParamsChanged = True
    End Sub

    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        If Not EventsAreEnabled Then Return
        ShowHideControls()
        InitGraphics(pbox_Scope)
        ParamsChanged = True
        ScopeUpdate()
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        SaveBufferTo(GetBuffersPath() + "LastBuffer.csv", True)
        Save_INI()
        Timer_30Hz.Stop()
        Timer_10Hz.Stop()
        btn_Run.Checked = False
        StartStopAcquisition(btn_Run.Checked)
    End Sub

    Private Sub Form1_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        LimitFormPosition(Me)
        ParamsChanged = True
    End Sub

    Private Sub btn_EditSlotNames_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_EditSlotNames.Click
        Me.Opacity = 0.7
        ProcessStartAndWait(Application.StartupPath + "\SlotNames.txt")
        Me.Opacity = 1
    End Sub

    Private Sub ShowHideControls()
        gbox_SlotNames.Visible = Me.Size.Width >= 800
        'btn_Interpolate.Visible = Me.Size.Width >= 705
    End Sub

    Private Sub PaintVerticalText(ByVal sender As Object, ByVal e As PaintEventArgs) Handles btn_SetZero.Paint
        Dim c As Control = CType(sender, Control)
        e.Graphics.RotateTransform(-90)
        e.Graphics.DrawString(c.Tag.ToString, c.Font, New SolidBrush(c.ForeColor), _
                              -c.Height + 2, 2, StringFormat.GenericDefault)
    End Sub


    ' ===================================================================================
    '  MenuStrip and ToolStrip accepting the first click
    '  If the form receives a WM_PARENTNOTIFY (528) message and is not focused 
    '  then the form is activated before to exec the message
    ' ===================================================================================
    Protected Overrides Sub WndProc(ByRef m As Message)
        If m.Msg = 528 AndAlso Not Me.Focused Then
            Me.Activate()
        End If
        MyBase.WndProc(m)
    End Sub

    ' ===================================================================================
    '   ToolStrip Gradients
    ' ===================================================================================
    Private Sub ToolStrip1_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles ToolStrip1.Paint
        Dim bounds As New Rectangle(0, 0, _
                                    ToolStrip1.Width, ToolStrip1.Height)
        Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
                                                       Color.White, _
                                                       Color.FromArgb(200, 200, 200), _
                                                       Drawing2D.LinearGradientMode.Vertical)
        e.Graphics.FillRectangle(brush, bounds)
    End Sub

    ' ===================================================================================
    '   ToolStrip PressedButton color
    ' ===================================================================================
    Class ToolStripButtonRenderer
        Inherits System.Windows.Forms.ToolStripProfessionalRenderer

        Protected Overrides Sub OnRenderButtonBackground(ByVal e As ToolStripItemRenderEventArgs)
            Dim btn As ToolStripButton = CType(e.Item, ToolStripButton)
            If btn IsNot Nothing AndAlso btn.Checked Then
                Dim bounds As Rectangle = New Rectangle(0, 0, e.Item.Width - 1, e.Item.Height - 1)
                Dim brush As New Drawing2D.LinearGradientBrush(bounds, _
                                                               Color.Gold, _
                                                               Color.FromArgb(250, 250, 250), _
                                                               Drawing2D.LinearGradientMode.Vertical)
                e.Graphics.FillRectangle(brush, bounds)
                e.Graphics.DrawRectangle(Pens.Orange, bounds)
            Else
                MyBase.OnRenderButtonBackground(e)
            End If
        End Sub
    End Class


    ' ===================================================================================
    '   ToolStrip Buttons
    ' ===================================================================================
    Private Sub btn_LoadBuffer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_LoadBuffer.Click
        LoadBuffer()
        FlushMouseEvents(Me)
        TriggerCh = 0
        UpdateTriggerButton()
        RefreshNeeded = True
    End Sub

    Private Sub FlushMouseEvents(ByVal frm As Form)
        frm.Enabled = False
        Application.DoEvents()
        frm.Enabled = True
    End Sub

    Private Sub btn_SaveBuffer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_SaveBuffer.Click
        SaveBuffer()
    End Sub
    Private Sub btn_SaveImage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_SaveImage.Click
        btn_SaveImage.Visible = False
        btn_SaveImage.Visible = True
        Me.Refresh()
        SaveImage_("SignalScope_", Me)
    End Sub

    Private Sub btn_Time_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_Time.Click
        ModuleScope.LargeMarkers = btn_Time.Checked
        RefreshNeeded = True
        ParamsChanged = True
    End Sub

    Private Sub btn_Interpolate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_Interpolate.Click
        ModuleScope.InterpolateEnabled = btn_Interpolate.Checked
        RefreshNeeded = True
        ParamsChanged = True
    End Sub

    Private Sub btn_Cursors_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_Cursors.Click
        ModuleScope.CursorsEnabled = btn_Cursors.Checked
        btn_Interpolate.Visible = Not btn_Cursors.Checked
        RefreshNeeded = True
        ParamsChanged = True
    End Sub

    Private Sub btn_Trigger_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_Trigger.Click
        TriggerCh = (TriggerCh + 1) Mod 5
        UpdateTriggerButton()
        ScopeUpdate()
        RefreshNeeded = True
    End Sub
    Private Sub UpdateTriggerButton()
        If TriggerCh = 0 Then
            btn_Trigger.Checked = False
            btn_Trigger.Text = "Trigger OFF"
        Else
            btn_Trigger.Checked = True
            btn_Trigger.Text = "Trigger CH" + TriggerCh.ToString
        End If
    End Sub

    Private Sub btn_Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btn_Run.Click
        txt_Delta.NumericValue = 0
        ModuleScope.StartStopAcquisition(btn_Run.Checked)
        RefreshNeeded = True
        ParamsChanged = True
    End Sub


    ' ====================================================================================================
    '   TIMERS
    ' ====================================================================================================
    Private Sub StartStopTimers()
        If Visible And WindowState <> FormWindowState.Minimized Then
            Timer_10Hz.Interval = 100
            Timer_10Hz.Start()
            Timer_30Hz.Interval = 33
            Timer_30Hz.Start()
            Timer_60Hz.Interval = 10
            Timer_60Hz.Start()
        Else
            Timer_10Hz.Stop()
            Timer_30Hz.Stop()
            Timer_60Hz.Stop()
        End If
    End Sub

    Private Sub Timer_10Hz_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer_10Hz.Tick
        If Not EventsAreEnabled Then Return
        If Not pbox_Scope.Visible Then Return
        If btn_Run.Checked OrElse RefreshNeeded Then
            ShowStatusBarValues()
        End If
        If ParamsChanged Then Save_INI()
        TestSlotFile()
    End Sub

    Private Sub Timer_30Hz_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer_30Hz.Tick
        If Not EventsAreEnabled Then Return
        If Not pbox_Scope.Visible Then Return
        If btn_Run.Checked OrElse RefreshNeeded Then
            RefreshNeeded = False
            ScopeUpdate()
        End If
    End Sub

    Private Sub Timer_60Hz_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer_60Hz.Tick
        If Not EventsAreEnabled Then Return
        ' ------------------------------------------------- test stop flag
        If StopFlag Then
            btn_Run.Checked = False
            StartStopAcquisition(btn_Run.Checked)
            StartStopTimers()
            ScopeUpdate()
        End If
        ' ------------------------------------------------- test start/stop Slot
        Static sw As Stopwatch = Nothing
        If txt_RunSlot.NumericValueInteger > 0 Then
            If Slots.ReadSlot_NoNan(txt_RunSlot.NumericValueInteger) <> 0 Then
                If Not btn_Run.Checked Then
                    btn_Run.Checked = True
                    StartStopAcquisition(True)
                    StartStopTimers()
                    ScopeUpdate()
                    Me.TopMost = True
                    Me.TopMost = False
                End If
                sw = Nothing
            Else
                If btn_Run.Checked Then
                    If sw Is Nothing Then
                        sw = New Stopwatch
                        sw.Start()
                    End If
                    If sw.ElapsedMilliseconds > txt_StopDelayMillisec.NumericValueInteger Then
                        sw = Nothing
                        btn_Run.Checked = False
                        StartStopAcquisition(False)
                        StartStopTimers()
                        ScopeUpdate()
                    End If
                End If
            End If
        End If
        ' ------------------------------------------------- test saveBuffer Slot
        Static SaveBufferEnabled As Boolean
        If txt_SaveBufferSlot.NumericValueInteger > 0 Then
            If Slots.ReadSlot_NoNan(txt_SaveBufferSlot.NumericValueInteger) > 500 Then
                If SaveBufferEnabled Then
                    SaveBufferEnabled = False
                    SaveBufferAuto()
                End If
            Else
                SaveBufferEnabled = True
            End If
        End If
        ' ------------------------------------------------- test saveImage Slot
        Static SaveImageEnabled As Boolean
        If txt_SaveImageSlot.NumericValueInteger > 0 Then
            If Slots.ReadSlot_NoNan(txt_SaveImageSlot.NumericValueInteger) > 500 Then
                If SaveImageEnabled Then
                    SaveImageEnabled = False
                    SaveImageAuto_("SignalScope_", Me)
                End If
            Else
                SaveImageEnabled = True
            End If
        End If
    End Sub


    ' ====================================================================================================
    '  Set DeltaTime by MouseDown and MouseMove on the Pbox_Scope
    ' ====================================================================================================
    Private oldx As Int32
    Private EditDeltaTime As Boolean
    Private EditCursor1 As Boolean
    Private EditCursor2 As Boolean
    Private Sub pbox_Scope_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbox_Scope.MouseDown
        If e.Button = MouseButtons.Left Then
            ' ------------------------------------------------------------- init edit cursors
            If ModuleScope.CursorsEnabled Then
                Dim x As Single = (e.X - 1.0F) / (pbox_Scope.ClientSize.Width - 2.0F)
                If Math.Abs(x - Cursor1) < 0.05 Then
                    EditCursor1 = True
                ElseIf Math.Abs(x - Cursor2) < 0.05 Then
                    EditCursor2 = True
                Else
                    ' ------------------------------------------------------------- init for delta time
                    oldx = e.X
                    EditDeltaTime = True
                End If
            Else
                ' ------------------------------------------------------------- init for delta time
                oldx = e.X
                EditDeltaTime = True
            End If
        End If
    End Sub

    Private Sub pbox_Scope_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbox_Scope.MouseMove
        If e.Button = MouseButtons.Left Or e.Button = MouseButtons.Right Then
            If EditCursor1 Or EditCursor2 Then
                Dim x As Single = e.X
                If x > pbox_Scope.ClientSize.Width - 2 Then x = pbox_Scope.ClientSize.Width - 2
                If x < 2 Then x = 2
                x = (x - 1.0F) / (pbox_Scope.ClientSize.Width - 2.0F)
                If EditCursor1 Then Cursor1 = x
                If EditCursor2 Then Cursor2 = x
                ScopeUpdate()
                pbox_Scope.Refresh()
                ParamsChanged = True
            End If
            If EditDeltaTime Then
                Dim delta As Int32 = e.X - oldx
                If My.Computer.Keyboard.ShiftKeyDown Then delta *= 100
                If My.Computer.Keyboard.CtrlKeyDown Then delta *= 10
                ModuleScope.UpdateDeltaTimeBy(delta)
                oldx = e.X
                EventsAreEnabled = False
                txt_Delta.Text = ModuleScope.GetDeltaTime.ToString("0")
                EventsAreEnabled = True
                RefreshNeeded = True
            End If
        End If
    End Sub

    Private Sub txt_Delta_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txt_Delta.TextChanged
        '
        If txt_Delta.Text = "0" Then
            txt_Delta.BackColor = Color.FromArgb(240, 240, 240)
            txt_Delta.BackColor_Over = Color.White
        Else
            txt_Delta.BackColor = Color.Orange
            txt_Delta.BackColor_Over = Color.FromArgb(255, 190, 100)
        End If
        '
        If Not EventsAreEnabled Then Return
        ModuleScope.SetDeltaTime(CSng(Val(txt_Delta.Text)))
        SetScopeParams()
        ParamsChanged = True
    End Sub

    Private Sub txt_Delta_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles txt_Delta.DoubleClick
        txt_Delta.Text = "0"
    End Sub

    Private Sub pbox_Scope_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbox_Scope.MouseUp
        EditDeltaTime = False
        EditCursor1 = False
        EditCursor2 = False
    End Sub

    ' ====================================================================================================
    '   Set BaseTime by MouseWheel on the Pbox_Scope
    '    ZOOM AT MOUSE POSITION 
    '      - NOT WORKING OK BECAUSE NOT PERFECT 
    '      - AND NOT GOOD BECAUSE MOVES THE "DELTA" AND LEAVES IT AS "NOT ZERO" VALUE
    ' ====================================================================================================
    'Private Sub pbox_Scope_MouseWheel(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbox_Scope.MouseWheel

    '    Dim oldbasetime As Single = GetBaseTime()

    '    EventsAreEnabled = False
    '    SetComboByWheel(cmb_BaseTime, e.Delta)
    '    Application.DoEvents()
    '    EventsAreEnabled = True

    '    Dim deltabasetime As Single = oldbasetime / GetBaseTime()
    '    Dim deltapixels As Int32

    '    If deltabasetime > 1 Then
    '        deltapixels = CInt(0.5 * (pbox_Scope.ClientSize.Width - e.Location.X) * deltabasetime)
    '    Else
    '        deltapixels = -CInt((pbox_Scope.ClientSize.Width - e.Location.X) * deltabasetime)
    '    End If

    '    EventsAreEnabled = False
    '    UpdateDeltaTimeBy(deltapixels)
    '    txt_Delta.Text = ModuleScope.GetDeltaTime.ToString("0")
    '    Application.DoEvents()
    '    EventsAreEnabled = True


    '    'EventsAreEnabled = False
    '    'ModuleScope.SetDeltaTime(ModuleScope.GetDeltaTime + deltapixels * oldbasetime / pbox_Scope.ClientSize.Width * 20)
    '    'txt_Delta.Text = ModuleScope.GetDeltaTime.ToString("0")
    '    'Application.DoEvents()
    '    'EventsAreEnabled = True

    '    RefreshNeeded = True
    'End Sub

    ' ====================================================================================================
    '   Set BaseTime by MouseWheel on the Pbox_Scope
    ' ====================================================================================================
    Private Sub pbox_Scope_MouseWheel(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles pbox_Scope.MouseWheel
        SetComboByWheel(cmb_BaseTime, e.Delta)
    End Sub

    Friend Sub SetComboByWheel(ByRef cmb As ComboBox, ByVal delta As Single)
        Static acc As Single = -1
        If acc < 0 Then acc = cmb.SelectedIndex
        acc += delta / 120.0F
        If acc < 0 Then acc = 0
        If acc > cmb.Items.Count - 1 Then acc = cmb.Items.Count - 1
        Combo_SetIndex(cmb, CInt(acc))
    End Sub

    ' ====================================================================================================
    '   SET ZERO
    ' ====================================================================================================
    Private Sub btn_SetZero_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_SetZero.Click
        SetCenterValue()
    End Sub

    ' ====================================================================================================
    '   COMBO Units1
    ' ====================================================================================================
    Private Sub cmb_Units1_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units1.DropDown
        LimitComboDropDownHeight(cmb_Units1)
    End Sub
    Private Sub cmb_Units1_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units1.DropDownClosed
        pbox_Scope.Focus()
    End Sub
    Private Sub cmb_Units1_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units1.SelectedIndexChanged
        ch1.SetOldValues()
        SetScopeParams()
        If EventsAreEnabled Then ch1.CorrectCenterValue(txt_Center1.NumericValue)
    End Sub

    ' ====================================================================================================
    '   COMBO Units2
    ' ====================================================================================================
    Private Sub cmb_Units2_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units2.DropDown
        LimitComboDropDownHeight(cmb_Units2)
    End Sub
    Private Sub cmb_Units2_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units2.DropDownClosed
        pbox_Scope.Focus()
    End Sub
    Private Sub cmb_Units2_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units2.SelectedIndexChanged
        ch2.SetOldValues()
        SetScopeParams()
        If EventsAreEnabled Then ch2.CorrectCenterValue(txt_Center2.NumericValue)
    End Sub

    ' ====================================================================================================
    '   COMBO Units2
    ' ====================================================================================================
    Private Sub cmb_Units3_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units3.DropDown
        LimitComboDropDownHeight(cmb_Units3)
    End Sub
    Private Sub cmb_Units3_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units3.DropDownClosed
        pbox_Scope.Focus()
    End Sub
    Private Sub cmb_Units3_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units3.SelectedIndexChanged
        ch3.SetOldValues()
        SetScopeParams()
        If EventsAreEnabled Then ch3.CorrectCenterValue(txt_Center3.NumericValue)
    End Sub

    ' ====================================================================================================
    '   COMBO Units2
    ' ====================================================================================================
    Private Sub cmb_Units4_DropDown(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units4.DropDown
        LimitComboDropDownHeight(cmb_Units3)
    End Sub
    Private Sub cmb_Units4_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units4.DropDownClosed
        pbox_Scope.Focus()
    End Sub
    Private Sub cmb_Units4_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_Units4.SelectedIndexChanged
        ch4.SetOldValues()
        SetScopeParams()
        If EventsAreEnabled Then ch4.CorrectCenterValue(txt_Center4.NumericValue)
    End Sub

    ' ====================================================================================================
    '   COMBO BaseTime
    ' ====================================================================================================
    Private Sub cmb_BaseTime_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_BaseTime.DropDownClosed
        pbox_Scope.Focus()
    End Sub
    Private Sub cmb_BaseTime_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_BaseTime.SelectedIndexChanged
        SetScopeParams()
        ModuleScope.LimitDeltaTime()
        txt_Delta.NumericValue = ModuleScope.GetDeltaTime
    End Sub

    ' ====================================================================================================
    '   COMBO cmb_StopIf
    ' ====================================================================================================
    Private Sub cmb_StopIf_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_StopIf.DropDownClosed
        pbox_Scope.Focus()
    End Sub
    Private Sub cmb_StopIf_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_StopIf.SelectedIndexChanged
        SetStopParams()
    End Sub

    ' ====================================================================================================
    '   TextBox StopTripPoint
    ' ====================================================================================================
    Private Sub txt_StopTripPoint_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txt_StopTripPoint.TextChanged
        If Not EventsAreEnabled Then Return
        SetStopParams()
    End Sub

    ' ====================================================================================================
    '   COMBO cmb_StopIf
    ' ====================================================================================================
    Private Sub cmb_StopAfterTimes_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_StopAfterTimes.DropDownClosed
        pbox_Scope.Focus()
    End Sub
    Private Sub cmb_StopAfterTimes_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmb_StopAfterTimes.SelectedIndexChanged
        SetStopParams()
    End Sub


    ' ====================================================================================================
    '   Channel params
    ' ====================================================================================================
    Private Sub txt_Slot1234_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txt_Slot1.TextChanged, _
                                                                                                             txt_Slot2.TextChanged, _
                                                                                                             txt_Slot3.TextChanged, _
                                                                                                             txt_Slot4.TextChanged
        If Not EventsAreEnabled Then Return
        SetSlotParams()
    End Sub
    Private Sub txt_Min1234_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txt_Min1.TextChanged, _
                                                                                                            txt_Min2.TextChanged, _
                                                                                                            txt_Min3.TextChanged, _
                                                                                                            txt_Min4.TextChanged
        If Not EventsAreEnabled Then Return
        SetScopeParams()
    End Sub
    Private Sub txt_Max1234_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txt_Max1.TextChanged, _
                                                                                                            txt_Max2.TextChanged, _
                                                                                                            txt_Max3.TextChanged, _
                                                                                                            txt_Max4.TextChanged
        If Not EventsAreEnabled Then Return
        SetScopeParams()
    End Sub
    Private Sub txt_Center1234_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txt_Center1.TextChanged, _
                                                                                                               txt_Center2.TextChanged, _
                                                                                                               txt_Center3.TextChanged, _
                                                                                                               txt_Center4.TextChanged
        If Not EventsAreEnabled Then Return
        SetScopeParams()
    End Sub
    Private Sub chk_AC1234_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chk_AC1.CheckedChanged, _
                                                                                                              chk_AC2.CheckedChanged, _
                                                                                                              chk_AC3.CheckedChanged, _
                                                                                                              chk_AC4.CheckedChanged
        If Not EventsAreEnabled Then Return
        SetScopeParams()
    End Sub
    Private Sub chk_EN1234_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chk_EN1.CheckedChanged, _
                                                                                                              chk_EN2.CheckedChanged, _
                                                                                                              chk_EN3.CheckedChanged, _
                                                                                                              chk_EN4.CheckedChanged
        If Not EventsAreEnabled Then Return
        SetScopeParams()
    End Sub

    Private Sub txt_BufferLen_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txt_BufferLen.TextChanged
        If Not EventsAreEnabled Then Return
        SetBufferLength()
        ParamsChanged = True
        RefreshNeeded = True
    End Sub

    Private Sub SetBufferLength()
        txt_BufferLen.MaxValue = FifoLen * 2 \ 60000
        BufferLenMS = StringToInt32(txt_BufferLen.Text) * 60000
        If BufferLenMS = 0 Then
            BufferLenMS = CInt(txt_BufferLen.MaxValue * 60000)
            ClearBufferEachRun = True
        Else
            ClearBufferEachRun = False
        End If
        txt_Delta.MaxValue = BufferLenMS
        ModuleScope.LimitDeltaTime()
        txt_Delta.NumericValue = ModuleScope.GetDeltaTime
    End Sub


    Private Sub txt_RunSlot_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txt_RunSlot.TextChanged
        ParamsChanged = True
    End Sub
    Private Sub txt_StopDelayMillisec_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txt_StopDelayMillisec.TextChanged

    End Sub

    Friend Sub SetSlotParams()
        If Not EventsAreEnabled Then Return
        Dim n As Int32
        ' --------------------------------------- slot 1
        If txt_Slot1.Text.Trim <> "" Then
            n = CInt(Val(txt_Slot1.Text))
        Else
            n = -1
        End If
        If n >= 0 Then
            If SlotNames(n) <> "" Then
                lbl_SlotName1.Text = "BLU = " + SlotNames(n)
            Else
                lbl_SlotName1.Text = "BLU = Slot " + n.ToString
            End If
        Else
            lbl_SlotName1.Text = "Blu trace is unused"
        End If
        ModuleScope.ch1.Slot = n
        ' --------------------------------------- slot 2
        If txt_Slot2.Text.Trim <> "" Then
            n = CInt(Val(txt_Slot2.Text))
        Else
            n = -1
        End If
        If n >= 0 Then
            If SlotNames(n) <> "" Then
                lbl_SlotName2.Text = "RED = " + SlotNames(n)
            Else
                lbl_SlotName2.Text = "RED = Slot " + n.ToString
            End If
        Else
            lbl_SlotName2.Text = "Red trace is unused"
        End If
        ModuleScope.ch2.Slot = n
        ' --------------------------------------- slot 3
        If txt_Slot3.Text.Trim <> "" Then
            n = CInt(Val(txt_Slot3.Text))
        Else
            n = -1
        End If
        If n >= 0 Then
            If SlotNames(n) <> "" Then
                lbl_SlotName3.Text = "GRN = " + SlotNames(n)
            Else
                lbl_SlotName3.Text = "GRN = Slot " + n.ToString
            End If
        Else
            lbl_SlotName3.Text = "Green trace is unused"
        End If
        ModuleScope.ch3.Slot = n
        ' --------------------------------------- slot 4
        If txt_Slot4.Text.Trim <> "" Then
            n = CInt(Val(txt_Slot4.Text))
        Else
            n = -1
        End If
        If n >= 0 Then
            If SlotNames(n) <> "" Then
                lbl_SlotName4.Text = "BLK = " + SlotNames(n)
            Else
                lbl_SlotName4.Text = "BLK = Slot " + n.ToString
            End If
        Else
            lbl_SlotName4.Text = "Black trace is unused"
        End If
        ModuleScope.ch4.Slot = n
        ' -----------------------------------------
        ParamsChanged = True
        RefreshNeeded = True
        ShowStatusBarValues()
        SetScaleLabels()
    End Sub

    Private Sub SetScopeParams()
        ' ------------------------------------------------------- 1
        If cmb_Units1.Text = "Min-Max" Then
            ModuleScope.ch1.Min = CInt(Val(txt_Min1.Text))
            ModuleScope.ch1.Max = CInt(Val(txt_Max1.Text))
            ModuleScope.ch1.vDiv = (ModuleScope.ch1.Max - ModuleScope.ch1.Min) / 10
            ch1.Value_Zero = (ch1.Max + ch1.Min) / 2.0F
            txt_Min1.Enabled = True
            txt_Max1.Enabled = True
            txt_Center1.Enabled = False
        Else
            ModuleScope.ch1.vDiv = CSng(Val(cmb_Units1.Text))
            ch1.Value_Zero = CSng(Val(txt_Center1.Text)) * ModuleScope.ch1.vDiv
            txt_Min1.Enabled = False
            txt_Max1.Enabled = False
            txt_Center1.Enabled = True
        End If
        ' ------------------------------------------------------- 2
        If cmb_Units2.Text = "Min-Max" Then
            ModuleScope.ch2.Min = CInt(Val(txt_Min2.Text))
            ModuleScope.ch2.Max = CInt(Val(txt_Max2.Text))
            ModuleScope.ch2.vDiv = (ModuleScope.ch2.Max - ModuleScope.ch2.Min) / 10
            ch2.Value_Zero = (ch2.Max + ch2.Min) / 2.0F
            txt_Min2.Enabled = True
            txt_Max2.Enabled = True
            txt_Center2.Enabled = False
        Else
            ModuleScope.ch2.vDiv = CSng(Val(cmb_Units2.Text))
            ch2.Value_Zero = CSng(Val(txt_Center2.Text)) * ModuleScope.ch2.vDiv
            txt_Min2.Enabled = False
            txt_Max2.Enabled = False
            txt_Center2.Enabled = True
        End If
        ' ------------------------------------------------------- 3
        If cmb_Units3.Text = "Min-Max" Then
            ModuleScope.ch3.Min = CInt(Val(txt_Min3.Text))
            ModuleScope.ch3.Max = CInt(Val(txt_Max3.Text))
            ModuleScope.ch3.vDiv = (ModuleScope.ch3.Max - ModuleScope.ch3.Min) / 10
            ch3.Value_Zero = (ch3.Max + ch3.Min) / 2.0F
            txt_Min3.Enabled = True
            txt_Max3.Enabled = True
            txt_Center3.Enabled = False
        Else
            ModuleScope.ch3.vDiv = CSng(Val(cmb_Units3.Text))
            ch3.Value_Zero = CSng(Val(txt_Center3.Text)) * ModuleScope.ch3.vDiv
            txt_Min3.Enabled = False
            txt_Max3.Enabled = False
            txt_Center3.Enabled = True
        End If
        ' ------------------------------------------------------- 4
        If cmb_Units4.Text = "Min-Max" Then
            ModuleScope.ch4.Min = CInt(Val(txt_Min4.Text))
            ModuleScope.ch4.Max = CInt(Val(txt_Max4.Text))
            ModuleScope.ch4.vDiv = (ModuleScope.ch4.Max - ModuleScope.ch4.Min) / 10
            ch4.Value_Zero = (ch4.Max + ch4.Min) / 2.0F
            txt_Min4.Enabled = True
            txt_Max4.Enabled = True
            txt_Center4.Enabled = False
        Else
            ModuleScope.ch4.vDiv = CSng(Val(cmb_Units4.Text))
            ch4.Value_Zero = CSng(Val(txt_Center4.Text)) * ModuleScope.ch4.vDiv
            txt_Min4.Enabled = False
            txt_Max4.Enabled = False
            txt_Center4.Enabled = True
        End If
        '
        ModuleScope.SetBaseTime(cmb_BaseTime.Text)
        '
        ModuleScope.ch1.RemoveDC = chk_AC1.Checked
        ModuleScope.ch2.RemoveDC = chk_AC2.Checked
        ModuleScope.ch3.RemoveDC = chk_AC3.Checked
        ModuleScope.ch4.RemoveDC = chk_AC4.Checked
        '
        ModuleScope.ch1.Enabled = chk_EN1.Checked
        ModuleScope.ch2.Enabled = chk_EN2.Checked
        ModuleScope.ch3.Enabled = chk_EN3.Checked
        ModuleScope.ch4.Enabled = chk_EN4.Checked
        '
        ParamsChanged = True
        '
        If EventsAreEnabled Then ScopeUpdate()
        pbox_Scope.Refresh()
        RefreshNeeded = False
        '
        ShowStatusBarValues()
        SetScaleLabels()
    End Sub

    Private Sub SetStopParams()
        If Not EventsAreEnabled Then Return
        ' -----------------------------------------
        ModuleScope.StopMethod = cmb_StopIf.SelectedIndex
        ModuleScope.StopTripPoint = CSng(Val(txt_StopTripPoint.Text))
        ModuleScope.StopNumEvents = Math.Max(CInt(Val(cmb_StopAfterTimes.Text)), 1)
        ' -----------------------------------------
        If ModuleScope.StopMethod > 0 Then
            txt_StopTripPoint.Visible = True
            lbl_Then.Visible = True
            cmb_StopAfterTimes.Visible = True
        Else
            txt_StopTripPoint.Visible = False
            lbl_Then.Visible = False
            cmb_StopAfterTimes.Visible = False
        End If
        ' -----------------------------------------
        ParamsChanged = True
        RefreshNeeded = True
        ShowStatusBarValues()
        SetScaleLabels()
    End Sub

    Private Sub SetCenterValue()
        EventsAreEnabled = False
        '
        If cmb_Units1.Text <> "Min-Max" Then
            ch1.Value_Zero = (ch1.ValueMax + ch1.ValueMin) / 2.0F
            txt_Center1.Text = (ch1.Value_Zero / ModuleScope.ch1.vDiv).ToString("0", GCI)
        End If
        If cmb_Units2.Text <> "Min-Max" Then
            ch2.Value_Zero = (ch2.ValueMax + ch2.ValueMin) / 2.0F
            txt_Center2.Text = (ch2.Value_Zero / ModuleScope.ch2.vDiv).ToString("0", GCI)
        End If
        If cmb_Units3.Text <> "Min-Max" Then
            ch3.Value_Zero = (ch3.ValueMax + ch3.ValueMin) / 2.0F
            txt_Center3.Text = (ch3.Value_Zero / ModuleScope.ch3.vDiv).ToString("0", GCI)
        End If
        If cmb_Units4.Text <> "Min-Max" Then
            ch4.Value_Zero = (ch4.ValueMax + ch4.ValueMin) / 2.0F
            txt_Center4.Text = (ch4.Value_Zero / ModuleScope.ch4.vDiv).ToString("0", GCI)
        End If
        ' ---------------------
        SetScaleLabels()
        ' ---------------------
        SetScopeParams()
        ' ---------------------
        EventsAreEnabled = True
        RefreshNeeded = True
    End Sub

    Private Sub SetScaleLabels()
        Dim FS1 As String = "0.#"
        Select Case cmb_Units1.Text
            Case "0.05", "0.02", "0.01" : FS1 = "0.##"
            Case "0.005", "0.002", "0.001" : FS1 = "0.###"
            Case "0.0005", "0.0002", "0.0001" : FS1 = "0.####"
        End Select
        Dim FS2 As String = "0.#"
        Select Case cmb_Units2.Text
            Case "0.05", "0.02", "0.01" : FS2 = "0.##"
            Case "0.005", "0.002", "0.001" : FS2 = "0.###"
            Case "0.0005", "0.0002", "0.0001" : FS2 = "0.####"
        End Select
        Dim FS3 As String = "0.#"
        Select Case cmb_Units3.Text
            Case "0.05", "0.02", "0.01" : FS3 = "0.##"
            Case "0.005", "0.002", "0.001" : FS3 = "0.###"
            Case "0.0005", "0.0002", "0.0001" : FS3 = "0.####"
        End Select
        Dim FS4 As String = "0.#"
        Select Case cmb_Units4.Text
            Case "0.05", "0.02", "0.01" : FS4 = "0.##"
            Case "0.005", "0.002", "0.001" : FS4 = "0.###"
            Case "0.0005", "0.0002", "0.0001" : FS4 = "0.####"
        End Select
        Label_Scale1a.Text = ""
        Label_Scale1b.Text = ""
        Label_Scale1c.Text = ""
        Label_Scale2a.Text = ""
        Label_Scale2b.Text = ""
        Label_Scale2c.Text = ""
        Label_Scale3a.Text = ""
        Label_Scale3b.Text = ""
        Label_Scale3c.Text = ""
        Label_Scale4a.Text = ""
        Label_Scale4b.Text = ""
        Label_Scale4c.Text = ""
        If ch1.Enabled AndAlso ch1.Slot > 0 Then
            Dim zv As Single = ch1.Value_Zero
            Label_Scale1a.Text = (zv + 5 * ch1.vDiv).ToString(FS1, GCI)
            Label_Scale1b.Text = zv.ToString(FS1, GCI)
            Label_Scale1c.Text = (zv - 5 * ch1.vDiv).ToString(FS1, GCI)
        End If
        If ch2.Enabled AndAlso ch2.Slot > 0 Then
            Dim zv As Single = ch2.Value_Zero
            Label_Scale2a.Text = (zv + 5 * ch2.vDiv).ToString(FS2, GCI)
            Label_Scale2b.Text = zv.ToString(FS2, GCI)
            Label_Scale2c.Text = (zv - 5 * ch2.vDiv).ToString(FS2, GCI)
        End If
        If ch3.Enabled AndAlso ch3.Slot > 0 Then
            Dim zv As Single = ch3.Value_Zero
            Label_Scale3a.Text = (zv + 5 * ch3.vDiv).ToString(FS3, GCI)
            Label_Scale3b.Text = zv.ToString(FS3, GCI)
            Label_Scale3c.Text = (zv - 5 * ch3.vDiv).ToString(FS3, GCI)
        End If
        If ch4.Enabled AndAlso ch4.Slot > 0 Then
            Dim zv As Single = ch4.Value_Zero
            Label_Scale4a.Text = (zv + 5 * ch4.vDiv).ToString(FS4, GCI)
            Label_Scale4b.Text = zv.ToString(FS4, GCI)
            Label_Scale4c.Text = (zv - 5 * ch4.vDiv).ToString(FS4, GCI)
        End If
        Label_Scale1a.Refresh()
        Label_Scale1b.Refresh()
        Label_Scale1c.Refresh()
        Label_Scale2a.Refresh()
        Label_Scale2b.Refresh()
        Label_Scale2c.Refresh()
        Label_Scale3a.Refresh()
        Label_Scale3b.Refresh()
        Label_Scale3c.Refresh()
        Label_Scale4a.Refresh()
        Label_Scale4b.Refresh()
        Label_Scale4c.Refresh()
    End Sub

    Private Sub ShowStatusBarValues()
        If ch1.Value = -99999999 And ch2.Value = -99999999 Then
            lbl_Value1.Text = ""
            lbl_Value2.Text = ""
            lbl_Value3.Text = ""
            lbl_Value4.Text = ""
            lbl_ValuePP1.Text = ""
            lbl_ValuePP2.Text = ""
            lbl_ValuePP3.Text = ""
            lbl_ValuePP4.Text = ""
            Return
        End If
        If ch1.Slot >= 0 Then
            With ch1
                lbl_Value1.Text = "Value = " + .Value.ToString(GCI)
                lbl_ValuePP1.Text = "Peak to peak = " + (.ValueMax - .ValueMin).ToString(GCI)
            End With
        End If
        If ch2.Slot >= 0 Then
            With ch2
                lbl_Value2.Text = "Value = " + .Value.ToString(GCI)
                lbl_ValuePP2.Text = "Peak to peak = " + (.ValueMax - .ValueMin).ToString(GCI)
            End With
        End If
        If ch3.Slot >= 0 Then
            With ch3
                lbl_Value3.Text = "Value = " + .Value.ToString(GCI)
                lbl_ValuePP3.Text = "Peak to peak = " + (.ValueMax - .ValueMin).ToString(GCI)
            End With
        End If
        If ch4.Slot >= 0 Then
            With ch4
                lbl_Value4.Text = "Value = " + .Value.ToString(GCI)
                lbl_ValuePP4.Text = "Peak to peak = " + (.ValueMax - .ValueMin).ToString(GCI)
            End With
        End If
    End Sub


End Class